<template>
	<div class="flex flex-col w-full gap-2">
		<div class="relative sync-msg-tip" v-show="showSyncMsg" @click="closeSyncMsg">
			<div class="absolute flex-center w-full text-black opacity-75 bottom-0.5">
				<div class="w-fit bg-gray-300 p-2 rounded-md shadow-lg">
					<div class="font-bold mb-2">{{ syncMsgContent.title }}</div>
					<span>正在处理</span><span class="text-blue-400">{{ syncMsgContent.question }}</span
					><span>提问，敬请等待！</span>
				</div>
			</div>
		</div>

		<SceneSwitch class="ml-5" :isHome="isHome" @change="groupTypeChange" />
		<div class="playInput hl_input rounded-[22px] input-border input-shadow w-full">
			<!-- 主场景 -->

			<div class="assembly flex">
				<el-button
					title="输入常用语标题，可快捷调用常用语"
					class="label flex items-center cursor-pointer"
					link
					@click="commonPhrasesClick"
				>
					<img src="/static/images/wave/PlugIn.png" width="34" height="34" class="set-icon box-border" />
				</el-button>
			</div>

			<div class="set-input">
				<div v-if="attachList?.length > 0" class="flex gap-3.5 w-full overflow-x-auto px-2 pb-2">
					<div
						v-for="(item, index) in attachList"
						:key="index"
						class="flex items-center gap-2 bg-[#f5f5f5] px-2 py-3 rounded-lg w-[220px] relative group"
						:class="{ 'cursor-pointer': item.type === 'table' }"
						@click="openAttachPreview(item)"
					>
						<template v-if="item.type === 'file'">
							<el-image
								:zoom-rate="1.2"
								fit="cover"
								class="w-[24px] rounded cursor-pointer"
								v-if="item.model.groupType === 'image'"
								:src="item.model.previewUrl"
								:preview-src-list="[item.model.previewUrl]"
							/>
							<div v-else class="ywifont !text-[24px] flex-0" :class="[`ywicon-${item.icon}`, item.iconClass]"></div>
							<div class="flex flex-col gap-0.5 w-full flex-auto">
								<div class="font-bold over-ellipsis w-full">{{ item.title }}</div>
								<div class="text-info text-sm over-ellipsis w-full">{{ `${item.model.type}，${item.model.size}` }}</div>
							</div>
						</template>
						<template v-if="item.type === 'table'">
							<div class="ywifont !text-[24px] flex-0" :class="[`ywicon-${item.icon}`, item.iconClass]"></div>
							<div class="flex flex-col gap-0.5 w-full flex-auto">
								<div class="font-bold over-ellipsis w-full">{{ item.title }}</div>
								<div class="text-info text-sm over-ellipsis w-full">{{ `业务表格，${item.model.values?.length} 条记录` }}</div>
							</div>
						</template>
						<div
							class="group-hover:visible invisible absolute right-0.5 top-0.5 bg-red-500 flex-center rounded-full p-0.5 cursor-pointer"
							@click.stop="deleteAttachInIndex(index)"
						>
							<span class="ywifont ywicon-guanbi text-white !text-[10px] font-bold"></span>
						</div>
					</div>
				</div>
				<el-input
					ref="inputRef"
					class="question-input relative align-bottom set-inputAnswer"
					type="textarea"
					resize="none"
					maxlength="1024"
					:autosize="{ minRows: 1, maxRows: 8 }"
					v-elInputFocus
					show-word-limit
					@keydown="keydownInput"
					@input="inputText"
					v-model="inputValue"
					placeholder="新建对话开始"
					clearable
				>
				</el-input>
				<InputTip ref="inputTipRef" :inputValue="inputValue" @updateInputValue="updateInputValue" :isHome="isHome" />
			</div>
			<div class="h100 flex items-end">
				<div class="upload_img space-y">
					<div class="imgbox cursor-pointer flex items-center">
						<el-button
							title="清除"
							class="cursor-pointer !text-gray-500"
							link
							style="margin-left: unset; margin-right: 0px"
							@click="clearTextarea"
							icon="ele-Close"
							v-if="inputValue"
						>
						</el-button>
						<div class="flex-items-center gap-2">
							<el-tooltip placement="top" content="关联业务表格">
								<div
									class="cursor-pointer size-[38px] relative !z-10 rounded flex-center hover:bg-[#f2f2f2]"
									@click="openBusinessTable"
								>
									<span class="ywifont ywicon-biaoge !text-[24px]"></span>
								</div>
							</el-tooltip>
							<el-tooltip placement="top" content="上传附件">
								<div class="cursor-pointer size-[38px] relative !z-10 rounded flex-center hover:bg-[#f2f2f2]" @click="openFileClick">
									<span class="ywifont ywicon-fujian !text-[25px] font-bold"></span>
								</div>
							</el-tooltip>

							<el-tooltip placement="top" content="停止生成" v-if="isTalking">
								<div class="cursor-pointer !ml-0 size-[38px] bg-[#1d86ff] relative !z-10 rounded-full flex-center" link>
									<div
										class="size-[38px] bg-black text-white stop-breathe box-content rounded-full flex-center"
										@click="emits('stopGenClick')"
									>
										<span class="ywifont ywicon-jieshu"></span>
									</div>
								</div>
							</el-tooltip>
							<el-tooltip v-else placement="top" content="发送">
								<div class="size-[38px] rounded-full bg-black flex-center" @click="emits('sendClick')">
									<img src="/static/images/wave/QueryImg.png" />
								</div>
							</el-tooltip>
						</div>
					</div>
				</div>
			</div>
			<VoicePage
				v-model:isShow="voicePageIsShow"
				v-show="voicePageIsShow"
				@submit="(cb) => emits('sendClick', cb)"
				@updateInputValue="updateInputValue"
				:isHome="isHome"
			/>
			<CommonPhrases
				v-model:isShow="isShowPhrase"
				v-show="isShowPhrase"
				:isHome="isHome"
				ref="commonPhraseRef"
				@updateInput="updateInputValue"
			/>
			<BusinessTable v-model="businessTableIsShow" @submit="submitBusinessTable" />
			<BusinessTablePreview :data="attachPreviewData" v-model="attachPreviewIsShow" />
		</div>
	</div>
</template>

<script setup lang="ts">
import type { InputInstance } from 'element-plus';
import { nextTick, onActivated, onDeactivated, ref } from 'vue';
import BusinessTable from './businessTable/index.vue';
import BusinessTablePreview from './businessTablePreview/index.vue';
import type { Attach } from './hook/useAttach';
import { useAttach } from './hook/useAttach';
import { useInputEvent } from './hook/useInputEvent';
import { useUploadFile } from './hook/useUploadFile';
import InputTip from './inputTip/index.vue';
import CommonPhrases from './phrase/CommonPhrases.vue';
import SceneSwitch from './SceneSwitch.vue';
import VoicePage from './voicePage/VoicePage.vue';
import { useCompRef } from '/@/utils/types';
import emitter from '/@/utils/mitt';

const emits = defineEmits(['sendClick', 'stopGenClick']);
const props = defineProps({
	isTalking: Boolean,
	isHome: Boolean,
	msgList: Array,
});

const voicePageIsShow = defineModel('voicePageIsShow', {
	type: Boolean,
	default: false,
});

const isShowPhrase = ref(false);
const inputValue = defineModel({
	type: String,
});
const inputRef = ref<InputInstance>(null);

const updateInputValue = (val) => {
	inputValue.value = val;
};

const { keydownInput, inputText, inputTipRef } = useInputEvent({
	props: props,
	emit: emits,
	inputValue: inputValue,
	inputRef: inputRef,
});

const clearTextarea = () => {
	inputValue.value = '';
};

const groupTypeChange = () => {
	inputRef.value.focus();
	commonPhraseRef.value.updatePhrase();
};

//#region ====================== 常用语功能 ======================
const commonPhraseRef = useCompRef(CommonPhrases);
// 常用语功能点击
const commonPhrasesClick = () => {
	isShowPhrase.value = true;
};

const addPhrase = (val) => {
	commonPhraseRef.value.addPhrase(val);
};

//#endregion
const showSyncMsg = ref(false);
const syncMsgDuration = 3000;
const syncMsgContent = ref({
	title: '',
	// content: '',
	question: '',
});
const resetSyncMsg = () => {
	syncMsgContent.value.title = '';
	syncMsgContent.value.question = '';
};
const closeSyncMsg = () => {
	showSyncMsg.value = false;
	nextTick(() => {
		resetSyncMsg();
	});
};
const showSyncTip = (data) => {
	showSyncMsg.value = true;
	syncMsgContent.value.title = `来自：${data?.from || '未知端'}消息`;
	const question = data?.question ? `“${data?.question}”` : '新的';
	syncMsgContent.value.question = question;
	// syncMsgContent.value.content = `正在处理${question}提问，请求等待！`;
	setTimeout(() => {
		closeSyncMsg();
	}, syncMsgDuration);
};

const { attachList, removeAttach, clearAttach } = useAttach();

const { openFileClick, deleteUploadFile } = useUploadFile({
	pastTarget: inputRef as any,
	attachFileList: attachList,
});
const deleteAttachInIndex = (index: number) => {
	const attach = attachList.value[index];
	if (attach.type === 'file') {
		deleteUploadFile(attach.model);
	}
	removeAttach(index);
};
const checkTableAttachExist = (data: any) => {
	const dataStr = JSON.stringify(data);
	const exist = attachList.value.find((item) => item.type === 'table' && JSON.stringify(item.model) === dataStr);
	if (exist) {
		return true;
	}
	return false;
};
const submitBusinessTable = (data) => {
	attachList.value.push(
		...data
			.filter((item) => !checkTableAttachExist(item))
			.map((item) => {
				return {
					title: item.title,
					type: 'table',
					model: item,
					icon: 'biaoge',
					iconClass: 'ywicon-biaoge text-[#c5e0ff]',
				};
			})
	);
};

//#region ====================== 业务表格 ======================
const businessTableIsShow = ref(false);
const openBusinessTable = () => {
	businessTableIsShow.value = true;
};
//#endregion

//#region ====================== 附件预览 ======================
const attachPreviewIsShow = ref(false);
const attachPreviewData = ref<Attach>();
const openAttachPreview = (item: Attach) => {
	if (item.type === 'file') return;
	attachPreviewIsShow.value = true;
	attachPreviewData.value = item;
};
//#endregion

const quoteAttach = (item: Attach) => {
	submitBusinessTable([item.model]);
};
onActivated(() => {
	emitter.on('quoteAttach', quoteAttach);
});

onDeactivated(() => {
	emitter.off('quoteAttach', quoteAttach);
});
defineExpose({
	addPhrase,
	showSyncTip,
	attachList,
	clearAttach,
});
</script>
<style scoped lang="scss">
@use './index.scss';
</style>
